package sun.security.util.math.intpoly;

import java.math.BigInteger;

public class P384OrderField extends IntegerPolynomial
{
    private static final int BITS_PER_LIMB = 28;
    private static final int NUM_LIMBS = 14;
    private static final int MAX_ADDS = 1;
    public static final BigInteger MODULUS;
    private static final long CARRY_ADD = 134217728L;
    private static final int LIMB_MASK = 268435455;
    
    public P384OrderField() {
        super(28, 14, 1, P384OrderField.MODULUS);
    }
    
    private static BigInteger evaluateModulus() {
        return BigInteger.valueOf(2L).pow(384).subtract(BigInteger.valueOf(54187661L)).subtract(BigInteger.valueOf(2L).pow(28).multiply(BigInteger.valueOf(20867411L))).add(BigInteger.valueOf(2L).pow(56).multiply(BigInteger.valueOf(10975981L))).add(BigInteger.valueOf(2L).pow(84).multiply(BigInteger.valueOf(14361739L))).subtract(BigInteger.valueOf(2L).pow(112).multiply(BigInteger.valueOf(35694566L))).subtract(BigInteger.valueOf(2L).pow(140).multiply(BigInteger.valueOf(132168845L))).subtract(BigInteger.valueOf(2L).pow(168).multiply(BigInteger.valueOf(3710130L)));
    }
    
    @Override
    protected void finalCarryReduceLast(long[] array) {
        long n = array[13] >> 20;
        int n2 = 13;
        array[n2] -= n << 20;
        long n3 = 54187661L * n;
        int n4 = 0;
        array[n4] += n3;
        long n5 = 20867411L * n;
        int n6 = 1;
        array[n6] += n5;
        long n7 = -10975981L * n;
        int n8 = 2;
        array[n8] += n7;
        long n9 = -14361739L * n;
        int n10 = 3;
        array[n10] += n9;
        long n11 = 35694566L * n;
        int n12 = 4;
        array[n12] += n11;
        long n13 = 132168845L * n;
        int n14 = 5;
        array[n14] += n13;
        long n15 = 3710130L * n;
        int n16 = 6;
        array[n16] += n15;
    }
    
    private void carryReduce(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14, long n15, long n16, long n17, long n18, long n19, long n20, long n21, long n22, long n23, long n24, long n25, long n26, long n27) {
        long n28 = 0L;
        long n29 = n + 134217728L >> 28;
        n -= n29 << 28;
        n2 += n29;
        long n30 = n2 + 134217728L >> 28;
        n2 -= n30 << 28;
        n3 += n30;
        long n31 = n3 + 134217728L >> 28;
        n3 -= n31 << 28;
        n4 += n31;
        long n32 = n4 + 134217728L >> 28;
        n4 -= n32 << 28;
        n5 += n32;
        long n33 = n5 + 134217728L >> 28;
        n5 -= n33 << 28;
        n6 += n33;
        long n34 = n6 + 134217728L >> 28;
        n6 -= n34 << 28;
        n7 += n34;
        long n35 = n7 + 134217728L >> 28;
        n7 -= n35 << 28;
        n8 += n35;
        long n36 = n8 + 134217728L >> 28;
        n8 -= n36 << 28;
        n9 += n36;
        long n37 = n9 + 134217728L >> 28;
        n9 -= n37 << 28;
        n10 += n37;
        long n38 = n10 + 134217728L >> 28;
        n10 -= n38 << 28;
        n11 += n38;
        long n39 = n11 + 134217728L >> 28;
        n11 -= n39 << 28;
        n12 += n39;
        long n40 = n12 + 134217728L >> 28;
        n12 -= n40 << 28;
        n13 += n40;
        long n41 = n13 + 134217728L >> 28;
        n13 -= n41 << 28;
        n14 += n41;
        long n42 = n14 + 134217728L >> 28;
        n14 -= n42 << 28;
        n15 += n42;
        long n43 = n15 + 134217728L >> 28;
        n15 -= n43 << 28;
        n16 += n43;
        long n44 = n16 + 134217728L >> 28;
        n16 -= n44 << 28;
        n17 += n44;
        long n45 = n17 + 134217728L >> 28;
        n17 -= n45 << 28;
        n18 += n45;
        long n46 = n18 + 134217728L >> 28;
        n18 -= n46 << 28;
        n19 += n46;
        long n47 = n19 + 134217728L >> 28;
        n19 -= n47 << 28;
        n20 += n47;
        long n48 = n20 + 134217728L >> 28;
        n20 -= n48 << 28;
        n21 += n48;
        long n49 = n21 + 134217728L >> 28;
        n21 -= n49 << 28;
        n22 += n49;
        long n50 = n22 + 134217728L >> 28;
        n22 -= n50 << 28;
        n23 += n50;
        long n51 = n23 + 134217728L >> 28;
        n23 -= n51 << 28;
        n24 += n51;
        long n52 = n24 + 134217728L >> 28;
        n24 -= n52 << 28;
        n25 += n52;
        long n53 = n25 + 134217728L >> 28;
        n25 -= n53 << 28;
        n26 += n53;
        long n54 = n26 + 134217728L >> 28;
        n26 -= n54 << 28;
        n27 += n54;
        long n55 = n27 + 134217728L >> 28;
        n27 -= n55 << 28;
        this.carryReduce0(array, n, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, n23, n24, n25, n26, n27, n28 + n55);
    }
    
    void carryReduce0(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14, long n15, long n16, long n17, long n18, long n19, long n20, long n21, long n22, long n23, long n24, long n25, long n26, long n27, long n28) {
        long n29 = 54187661L * n28;
        n14 += (n29 << 8 & 0xFFFFFFFL);
        n15 += n29 >> 20;
        long n30 = 20867411L * n28;
        n15 += (n30 << 8 & 0xFFFFFFFL);
        n16 += n30 >> 20;
        long n31 = -10975981L * n28;
        n16 += (n31 << 8 & 0xFFFFFFFL);
        n17 += n31 >> 20;
        long n32 = -14361739L * n28;
        n17 += (n32 << 8 & 0xFFFFFFFL);
        n18 += n32 >> 20;
        long n33 = 35694566L * n28;
        n18 += (n33 << 8 & 0xFFFFFFFL);
        n19 += n33 >> 20;
        long n34 = 132168845L * n28;
        n19 += (n34 << 8 & 0xFFFFFFFL);
        n20 += n34 >> 20;
        long n35 = 3710130L * n28;
        n20 += (n35 << 8 & 0xFFFFFFFL);
        n21 += n35 >> 20;
        long n36 = 54187661L * n27;
        n13 += (n36 << 8 & 0xFFFFFFFL);
        n14 += n36 >> 20;
        long n37 = 20867411L * n27;
        n14 += (n37 << 8 & 0xFFFFFFFL);
        n15 += n37 >> 20;
        long n38 = -10975981L * n27;
        n15 += (n38 << 8 & 0xFFFFFFFL);
        n16 += n38 >> 20;
        long n39 = -14361739L * n27;
        n16 += (n39 << 8 & 0xFFFFFFFL);
        n17 += n39 >> 20;
        long n40 = 35694566L * n27;
        n17 += (n40 << 8 & 0xFFFFFFFL);
        n18 += n40 >> 20;
        long n41 = 132168845L * n27;
        n18 += (n41 << 8 & 0xFFFFFFFL);
        n19 += n41 >> 20;
        long n42 = 3710130L * n27;
        n19 += (n42 << 8 & 0xFFFFFFFL);
        n20 += n42 >> 20;
        long n43 = 54187661L * n26;
        n12 += (n43 << 8 & 0xFFFFFFFL);
        n13 += n43 >> 20;
        long n44 = 20867411L * n26;
        n13 += (n44 << 8 & 0xFFFFFFFL);
        n14 += n44 >> 20;
        long n45 = -10975981L * n26;
        n14 += (n45 << 8 & 0xFFFFFFFL);
        n15 += n45 >> 20;
        long n46 = -14361739L * n26;
        n15 += (n46 << 8 & 0xFFFFFFFL);
        n16 += n46 >> 20;
        long n47 = 35694566L * n26;
        n16 += (n47 << 8 & 0xFFFFFFFL);
        n17 += n47 >> 20;
        long n48 = 132168845L * n26;
        n17 += (n48 << 8 & 0xFFFFFFFL);
        n18 += n48 >> 20;
        long n49 = 3710130L * n26;
        n18 += (n49 << 8 & 0xFFFFFFFL);
        n19 += n49 >> 20;
        long n50 = 54187661L * n25;
        n11 += (n50 << 8 & 0xFFFFFFFL);
        n12 += n50 >> 20;
        long n51 = 20867411L * n25;
        n12 += (n51 << 8 & 0xFFFFFFFL);
        n13 += n51 >> 20;
        long n52 = -10975981L * n25;
        n13 += (n52 << 8 & 0xFFFFFFFL);
        n14 += n52 >> 20;
        long n53 = -14361739L * n25;
        n14 += (n53 << 8 & 0xFFFFFFFL);
        n15 += n53 >> 20;
        long n54 = 35694566L * n25;
        n15 += (n54 << 8 & 0xFFFFFFFL);
        n16 += n54 >> 20;
        long n55 = 132168845L * n25;
        n16 += (n55 << 8 & 0xFFFFFFFL);
        n17 += n55 >> 20;
        long n56 = 3710130L * n25;
        n17 += (n56 << 8 & 0xFFFFFFFL);
        n18 += n56 >> 20;
        long n57 = 54187661L * n24;
        n10 += (n57 << 8 & 0xFFFFFFFL);
        n11 += n57 >> 20;
        long n58 = 20867411L * n24;
        n11 += (n58 << 8 & 0xFFFFFFFL);
        n12 += n58 >> 20;
        long n59 = -10975981L * n24;
        n12 += (n59 << 8 & 0xFFFFFFFL);
        n13 += n59 >> 20;
        long n60 = -14361739L * n24;
        n13 += (n60 << 8 & 0xFFFFFFFL);
        n14 += n60 >> 20;
        long n61 = 35694566L * n24;
        n14 += (n61 << 8 & 0xFFFFFFFL);
        n15 += n61 >> 20;
        long n62 = 132168845L * n24;
        n15 += (n62 << 8 & 0xFFFFFFFL);
        n16 += n62 >> 20;
        long n63 = 3710130L * n24;
        n16 += (n63 << 8 & 0xFFFFFFFL);
        n17 += n63 >> 20;
        long n64 = 54187661L * n23;
        n9 += (n64 << 8 & 0xFFFFFFFL);
        n10 += n64 >> 20;
        long n65 = 20867411L * n23;
        n10 += (n65 << 8 & 0xFFFFFFFL);
        n11 += n65 >> 20;
        long n66 = -10975981L * n23;
        n11 += (n66 << 8 & 0xFFFFFFFL);
        n12 += n66 >> 20;
        long n67 = -14361739L * n23;
        n12 += (n67 << 8 & 0xFFFFFFFL);
        n13 += n67 >> 20;
        long n68 = 35694566L * n23;
        n13 += (n68 << 8 & 0xFFFFFFFL);
        n14 += n68 >> 20;
        long n69 = 132168845L * n23;
        n14 += (n69 << 8 & 0xFFFFFFFL);
        n15 += n69 >> 20;
        long n70 = 3710130L * n23;
        n15 += (n70 << 8 & 0xFFFFFFFL);
        n16 += n70 >> 20;
        long n71 = 54187661L * n22;
        n8 += (n71 << 8 & 0xFFFFFFFL);
        n9 += n71 >> 20;
        long n72 = 20867411L * n22;
        n9 += (n72 << 8 & 0xFFFFFFFL);
        n10 += n72 >> 20;
        long n73 = -10975981L * n22;
        n10 += (n73 << 8 & 0xFFFFFFFL);
        n11 += n73 >> 20;
        long n74 = -14361739L * n22;
        n11 += (n74 << 8 & 0xFFFFFFFL);
        n12 += n74 >> 20;
        long n75 = 35694566L * n22;
        n12 += (n75 << 8 & 0xFFFFFFFL);
        n13 += n75 >> 20;
        long n76 = 132168845L * n22;
        n13 += (n76 << 8 & 0xFFFFFFFL);
        n14 += n76 >> 20;
        long n77 = 3710130L * n22;
        n14 += (n77 << 8 & 0xFFFFFFFL);
        n15 += n77 >> 20;
        long n78 = 54187661L * n21;
        n7 += (n78 << 8 & 0xFFFFFFFL);
        n8 += n78 >> 20;
        long n79 = 20867411L * n21;
        n8 += (n79 << 8 & 0xFFFFFFFL);
        n9 += n79 >> 20;
        long n80 = -10975981L * n21;
        n9 += (n80 << 8 & 0xFFFFFFFL);
        n10 += n80 >> 20;
        long n81 = -14361739L * n21;
        n10 += (n81 << 8 & 0xFFFFFFFL);
        n11 += n81 >> 20;
        long n82 = 35694566L * n21;
        n11 += (n82 << 8 & 0xFFFFFFFL);
        n12 += n82 >> 20;
        long n83 = 132168845L * n21;
        n12 += (n83 << 8 & 0xFFFFFFFL);
        n13 += n83 >> 20;
        long n84 = 3710130L * n21;
        n13 += (n84 << 8 & 0xFFFFFFFL);
        n14 += n84 >> 20;
        long n85 = 54187661L * n20;
        n6 += (n85 << 8 & 0xFFFFFFFL);
        n7 += n85 >> 20;
        long n86 = 20867411L * n20;
        n7 += (n86 << 8 & 0xFFFFFFFL);
        n8 += n86 >> 20;
        long n87 = -10975981L * n20;
        n8 += (n87 << 8 & 0xFFFFFFFL);
        n9 += n87 >> 20;
        long n88 = -14361739L * n20;
        n9 += (n88 << 8 & 0xFFFFFFFL);
        n10 += n88 >> 20;
        long n89 = 35694566L * n20;
        n10 += (n89 << 8 & 0xFFFFFFFL);
        n11 += n89 >> 20;
        long n90 = 132168845L * n20;
        n11 += (n90 << 8 & 0xFFFFFFFL);
        n12 += n90 >> 20;
        long n91 = 3710130L * n20;
        n12 += (n91 << 8 & 0xFFFFFFFL);
        n13 += n91 >> 20;
        long n92 = 54187661L * n19;
        n5 += (n92 << 8 & 0xFFFFFFFL);
        n6 += n92 >> 20;
        long n93 = 20867411L * n19;
        n6 += (n93 << 8 & 0xFFFFFFFL);
        n7 += n93 >> 20;
        long n94 = -10975981L * n19;
        n7 += (n94 << 8 & 0xFFFFFFFL);
        n8 += n94 >> 20;
        long n95 = -14361739L * n19;
        n8 += (n95 << 8 & 0xFFFFFFFL);
        n9 += n95 >> 20;
        long n96 = 35694566L * n19;
        n9 += (n96 << 8 & 0xFFFFFFFL);
        n10 += n96 >> 20;
        long n97 = 132168845L * n19;
        n10 += (n97 << 8 & 0xFFFFFFFL);
        n11 += n97 >> 20;
        long n98 = 3710130L * n19;
        n11 += (n98 << 8 & 0xFFFFFFFL);
        n12 += n98 >> 20;
        long n99 = 54187661L * n18;
        n4 += (n99 << 8 & 0xFFFFFFFL);
        n5 += n99 >> 20;
        long n100 = 20867411L * n18;
        n5 += (n100 << 8 & 0xFFFFFFFL);
        n6 += n100 >> 20;
        long n101 = -10975981L * n18;
        n6 += (n101 << 8 & 0xFFFFFFFL);
        n7 += n101 >> 20;
        long n102 = -14361739L * n18;
        n7 += (n102 << 8 & 0xFFFFFFFL);
        n8 += n102 >> 20;
        long n103 = 35694566L * n18;
        n8 += (n103 << 8 & 0xFFFFFFFL);
        n9 += n103 >> 20;
        long n104 = 132168845L * n18;
        n9 += (n104 << 8 & 0xFFFFFFFL);
        n10 += n104 >> 20;
        long n105 = 3710130L * n18;
        n10 += (n105 << 8 & 0xFFFFFFFL);
        n11 += n105 >> 20;
        long n106 = 54187661L * n17;
        n3 += (n106 << 8 & 0xFFFFFFFL);
        n4 += n106 >> 20;
        long n107 = 20867411L * n17;
        n4 += (n107 << 8 & 0xFFFFFFFL);
        n5 += n107 >> 20;
        long n108 = -10975981L * n17;
        n5 += (n108 << 8 & 0xFFFFFFFL);
        n6 += n108 >> 20;
        long n109 = -14361739L * n17;
        n6 += (n109 << 8 & 0xFFFFFFFL);
        n7 += n109 >> 20;
        long n110 = 35694566L * n17;
        n7 += (n110 << 8 & 0xFFFFFFFL);
        n8 += n110 >> 20;
        long n111 = 132168845L * n17;
        n8 += (n111 << 8 & 0xFFFFFFFL);
        n9 += n111 >> 20;
        long n112 = 3710130L * n17;
        n9 += (n112 << 8 & 0xFFFFFFFL);
        n10 += n112 >> 20;
        long n113 = 54187661L * n16;
        n2 += (n113 << 8 & 0xFFFFFFFL);
        n3 += n113 >> 20;
        long n114 = 20867411L * n16;
        n3 += (n114 << 8 & 0xFFFFFFFL);
        n4 += n114 >> 20;
        long n115 = -10975981L * n16;
        n4 += (n115 << 8 & 0xFFFFFFFL);
        n5 += n115 >> 20;
        long n116 = -14361739L * n16;
        n5 += (n116 << 8 & 0xFFFFFFFL);
        n6 += n116 >> 20;
        long n117 = 35694566L * n16;
        n6 += (n117 << 8 & 0xFFFFFFFL);
        n7 += n117 >> 20;
        long n118 = 132168845L * n16;
        n7 += (n118 << 8 & 0xFFFFFFFL);
        n8 += n118 >> 20;
        long n119 = 3710130L * n16;
        n8 += (n119 << 8 & 0xFFFFFFFL);
        n9 += n119 >> 20;
        long n120 = 54187661L * n15;
        n += (n120 << 8 & 0xFFFFFFFL);
        n2 += n120 >> 20;
        long n121 = 20867411L * n15;
        n2 += (n121 << 8 & 0xFFFFFFFL);
        n3 += n121 >> 20;
        long n122 = -10975981L * n15;
        n3 += (n122 << 8 & 0xFFFFFFFL);
        n4 += n122 >> 20;
        long n123 = -14361739L * n15;
        n4 += (n123 << 8 & 0xFFFFFFFL);
        n5 += n123 >> 20;
        long n124 = 35694566L * n15;
        n5 += (n124 << 8 & 0xFFFFFFFL);
        n6 += n124 >> 20;
        long n125 = 132168845L * n15;
        n6 += (n125 << 8 & 0xFFFFFFFL);
        n7 += n125 >> 20;
        long n126 = 3710130L * n15;
        n7 += (n126 << 8 & 0xFFFFFFFL);
        n8 += n126 >> 20;
        n15 = 0L;
        this.carryReduce1(array, n, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, n23, n24, n25, n26, n27, n28);
    }
    
    void carryReduce1(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14, long n15, long n16, long n17, long n18, long n19, long n20, long n21, long n22, long n23, long n24, long n25, long n26, long n27, long n28) {
        long n29 = n + 134217728L >> 28;
        n -= n29 << 28;
        n2 += n29;
        long n30 = n2 + 134217728L >> 28;
        n2 -= n30 << 28;
        n3 += n30;
        long n31 = n3 + 134217728L >> 28;
        n3 -= n31 << 28;
        n4 += n31;
        long n32 = n4 + 134217728L >> 28;
        n4 -= n32 << 28;
        n5 += n32;
        long n33 = n5 + 134217728L >> 28;
        n5 -= n33 << 28;
        n6 += n33;
        long n34 = n6 + 134217728L >> 28;
        n6 -= n34 << 28;
        n7 += n34;
        long n35 = n7 + 134217728L >> 28;
        n7 -= n35 << 28;
        n8 += n35;
        long n36 = n8 + 134217728L >> 28;
        n8 -= n36 << 28;
        n9 += n36;
        long n37 = n9 + 134217728L >> 28;
        n9 -= n37 << 28;
        n10 += n37;
        long n38 = n10 + 134217728L >> 28;
        n10 -= n38 << 28;
        n11 += n38;
        long n39 = n11 + 134217728L >> 28;
        n11 -= n39 << 28;
        n12 += n39;
        long n40 = n12 + 134217728L >> 28;
        n12 -= n40 << 28;
        n13 += n40;
        long n41 = n13 + 134217728L >> 28;
        n13 -= n41 << 28;
        n14 += n41;
        long n42 = n14 + 134217728L >> 28;
        n14 -= n42 << 28;
        n15 += n42;
        this.carryReduce2(array, n, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14, n15, n16, n17, n18, n19, n20, n21, n22, n23, n24, n25, n26, n27, n28);
    }
    
    void carryReduce2(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14, long n15, long n16, long n17, long n18, long n19, long n20, long n21, long n22, long n23, long n24, long n25, long n26, long n27, long n28) {
        long n29 = 54187661L * n15;
        n += (n29 << 8 & 0xFFFFFFFL);
        n2 += n29 >> 20;
        long n30 = 20867411L * n15;
        n2 += (n30 << 8 & 0xFFFFFFFL);
        n3 += n30 >> 20;
        long n31 = -10975981L * n15;
        n3 += (n31 << 8 & 0xFFFFFFFL);
        n4 += n31 >> 20;
        long n32 = -14361739L * n15;
        n4 += (n32 << 8 & 0xFFFFFFFL);
        n5 += n32 >> 20;
        long n33 = 35694566L * n15;
        n5 += (n33 << 8 & 0xFFFFFFFL);
        n6 += n33 >> 20;
        long n34 = 132168845L * n15;
        n6 += (n34 << 8 & 0xFFFFFFFL);
        n7 += n34 >> 20;
        long n35 = 3710130L * n15;
        n7 += (n35 << 8 & 0xFFFFFFFL);
        n8 += n35 >> 20;
        long n36 = n + 134217728L >> 28;
        n -= n36 << 28;
        n2 += n36;
        long n37 = n2 + 134217728L >> 28;
        n2 -= n37 << 28;
        n3 += n37;
        long n38 = n3 + 134217728L >> 28;
        n3 -= n38 << 28;
        n4 += n38;
        long n39 = n4 + 134217728L >> 28;
        n4 -= n39 << 28;
        n5 += n39;
        long n40 = n5 + 134217728L >> 28;
        n5 -= n40 << 28;
        n6 += n40;
        long n41 = n6 + 134217728L >> 28;
        n6 -= n41 << 28;
        n7 += n41;
        long n42 = n7 + 134217728L >> 28;
        n7 -= n42 << 28;
        n8 += n42;
        long n43 = n8 + 134217728L >> 28;
        n8 -= n43 << 28;
        n9 += n43;
        long n44 = n9 + 134217728L >> 28;
        n9 -= n44 << 28;
        n10 += n44;
        long n45 = n10 + 134217728L >> 28;
        n10 -= n45 << 28;
        n11 += n45;
        long n46 = n11 + 134217728L >> 28;
        n11 -= n46 << 28;
        n12 += n46;
        long n47 = n12 + 134217728L >> 28;
        n12 -= n47 << 28;
        n13 += n47;
        long n48 = n13 + 134217728L >> 28;
        n13 -= n48 << 28;
        n14 += n48;
        array[0] = n;
        array[1] = n2;
        array[2] = n3;
        array[3] = n4;
        array[4] = n5;
        array[5] = n6;
        array[6] = n7;
        array[7] = n8;
        array[8] = n9;
        array[9] = n10;
        array[10] = n11;
        array[11] = n12;
        array[12] = n13;
        array[13] = n14;
    }
    
    private void carryReduce(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14) {
        long n15 = 0L;
        long n16 = n + 134217728L >> 28;
        n -= n16 << 28;
        n2 += n16;
        long n17 = n2 + 134217728L >> 28;
        n2 -= n17 << 28;
        n3 += n17;
        long n18 = n3 + 134217728L >> 28;
        n3 -= n18 << 28;
        n4 += n18;
        long n19 = n4 + 134217728L >> 28;
        n4 -= n19 << 28;
        n5 += n19;
        long n20 = n5 + 134217728L >> 28;
        n5 -= n20 << 28;
        n6 += n20;
        long n21 = n6 + 134217728L >> 28;
        n6 -= n21 << 28;
        n7 += n21;
        long n22 = n7 + 134217728L >> 28;
        n7 -= n22 << 28;
        n8 += n22;
        long n23 = n8 + 134217728L >> 28;
        n8 -= n23 << 28;
        n9 += n23;
        long n24 = n9 + 134217728L >> 28;
        n9 -= n24 << 28;
        n10 += n24;
        long n25 = n10 + 134217728L >> 28;
        n10 -= n25 << 28;
        n11 += n25;
        long n26 = n11 + 134217728L >> 28;
        n11 -= n26 << 28;
        n12 += n26;
        long n27 = n12 + 134217728L >> 28;
        n12 -= n27 << 28;
        n13 += n27;
        long n28 = n13 + 134217728L >> 28;
        n13 -= n28 << 28;
        n14 += n28;
        long n29 = n14 + 134217728L >> 28;
        n14 -= n29 << 28;
        this.carryReduce0(array, n, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12, n13, n14, n15 + n29);
    }
    
    void carryReduce0(long[] array, long n, long n2, long n3, long n4, long n5, long n6, long n7, long n8, long n9, long n10, long n11, long n12, long n13, long n14, long n15) {
        long n16 = 54187661L * n15;
        n += (n16 << 8 & 0xFFFFFFFL);
        n2 += n16 >> 20;
        long n17 = 20867411L * n15;
        n2 += (n17 << 8 & 0xFFFFFFFL);
        n3 += n17 >> 20;
        long n18 = -10975981L * n15;
        n3 += (n18 << 8 & 0xFFFFFFFL);
        n4 += n18 >> 20;
        long n19 = -14361739L * n15;
        n4 += (n19 << 8 & 0xFFFFFFFL);
        n5 += n19 >> 20;
        long n20 = 35694566L * n15;
        n5 += (n20 << 8 & 0xFFFFFFFL);
        n6 += n20 >> 20;
        long n21 = 132168845L * n15;
        n6 += (n21 << 8 & 0xFFFFFFFL);
        n7 += n21 >> 20;
        long n22 = 3710130L * n15;
        n7 += (n22 << 8 & 0xFFFFFFFL);
        n8 += n22 >> 20;
        long n23 = n + 134217728L >> 28;
        n -= n23 << 28;
        n2 += n23;
        long n24 = n2 + 134217728L >> 28;
        n2 -= n24 << 28;
        n3 += n24;
        long n25 = n3 + 134217728L >> 28;
        n3 -= n25 << 28;
        n4 += n25;
        long n26 = n4 + 134217728L >> 28;
        n4 -= n26 << 28;
        n5 += n26;
        long n27 = n5 + 134217728L >> 28;
        n5 -= n27 << 28;
        n6 += n27;
        long n28 = n6 + 134217728L >> 28;
        n6 -= n28 << 28;
        n7 += n28;
        long n29 = n7 + 134217728L >> 28;
        n7 -= n29 << 28;
        n8 += n29;
        long n30 = n8 + 134217728L >> 28;
        n8 -= n30 << 28;
        n9 += n30;
        long n31 = n9 + 134217728L >> 28;
        n9 -= n31 << 28;
        n10 += n31;
        long n32 = n10 + 134217728L >> 28;
        n10 -= n32 << 28;
        n11 += n32;
        long n33 = n11 + 134217728L >> 28;
        n11 -= n33 << 28;
        n12 += n33;
        long n34 = n12 + 134217728L >> 28;
        n12 -= n34 << 28;
        n13 += n34;
        long n35 = n13 + 134217728L >> 28;
        n13 -= n35 << 28;
        n14 += n35;
        array[0] = n;
        array[1] = n2;
        array[2] = n3;
        array[3] = n4;
        array[4] = n5;
        array[5] = n6;
        array[6] = n7;
        array[7] = n8;
        array[8] = n9;
        array[9] = n10;
        array[10] = n11;
        array[11] = n12;
        array[12] = n13;
        array[13] = n14;
    }
    
    @Override
    protected void mult(long[] array, long[] array2, long[] array3) {
        this.carryReduce(array3, array[0] * array2[0], array[0] * array2[1] + array[1] * array2[0], array[0] * array2[2] + array[1] * array2[1] + array[2] * array2[0], array[0] * array2[3] + array[1] * array2[2] + array[2] * array2[1] + array[3] * array2[0], array[0] * array2[4] + array[1] * array2[3] + array[2] * array2[2] + array[3] * array2[1] + array[4] * array2[0], array[0] * array2[5] + array[1] * array2[4] + array[2] * array2[3] + array[3] * array2[2] + array[4] * array2[1] + array[5] * array2[0], array[0] * array2[6] + array[1] * array2[5] + array[2] * array2[4] + array[3] * array2[3] + array[4] * array2[2] + array[5] * array2[1] + array[6] * array2[0], array[0] * array2[7] + array[1] * array2[6] + array[2] * array2[5] + array[3] * array2[4] + array[4] * array2[3] + array[5] * array2[2] + array[6] * array2[1] + array[7] * array2[0], array[0] * array2[8] + array[1] * array2[7] + array[2] * array2[6] + array[3] * array2[5] + array[4] * array2[4] + array[5] * array2[3] + array[6] * array2[2] + array[7] * array2[1] + array[8] * array2[0], array[0] * array2[9] + array[1] * array2[8] + array[2] * array2[7] + array[3] * array2[6] + array[4] * array2[5] + array[5] * array2[4] + array[6] * array2[3] + array[7] * array2[2] + array[8] * array2[1] + array[9] * array2[0], array[0] * array2[10] + array[1] * array2[9] + array[2] * array2[8] + array[3] * array2[7] + array[4] * array2[6] + array[5] * array2[5] + array[6] * array2[4] + array[7] * array2[3] + array[8] * array2[2] + array[9] * array2[1] + array[10] * array2[0], array[0] * array2[11] + array[1] * array2[10] + array[2] * array2[9] + array[3] * array2[8] + array[4] * array2[7] + array[5] * array2[6] + array[6] * array2[5] + array[7] * array2[4] + array[8] * array2[3] + array[9] * array2[2] + array[10] * array2[1] + array[11] * array2[0], array[0] * array2[12] + array[1] * array2[11] + array[2] * array2[10] + array[3] * array2[9] + array[4] * array2[8] + array[5] * array2[7] + array[6] * array2[6] + array[7] * array2[5] + array[8] * array2[4] + array[9] * array2[3] + array[10] * array2[2] + array[11] * array2[1] + array[12] * array2[0], array[0] * array2[13] + array[1] * array2[12] + array[2] * array2[11] + array[3] * array2[10] + array[4] * array2[9] + array[5] * array2[8] + array[6] * array2[7] + array[7] * array2[6] + array[8] * array2[5] + array[9] * array2[4] + array[10] * array2[3] + array[11] * array2[2] + array[12] * array2[1] + array[13] * array2[0], array[1] * array2[13] + array[2] * array2[12] + array[3] * array2[11] + array[4] * array2[10] + array[5] * array2[9] + array[6] * array2[8] + array[7] * array2[7] + array[8] * array2[6] + array[9] * array2[5] + array[10] * array2[4] + array[11] * array2[3] + array[12] * array2[2] + array[13] * array2[1], array[2] * array2[13] + array[3] * array2[12] + array[4] * array2[11] + array[5] * array2[10] + array[6] * array2[9] + array[7] * array2[8] + array[8] * array2[7] + array[9] * array2[6] + array[10] * array2[5] + array[11] * array2[4] + array[12] * array2[3] + array[13] * array2[2], array[3] * array2[13] + array[4] * array2[12] + array[5] * array2[11] + array[6] * array2[10] + array[7] * array2[9] + array[8] * array2[8] + array[9] * array2[7] + array[10] * array2[6] + array[11] * array2[5] + array[12] * array2[4] + array[13] * array2[3], array[4] * array2[13] + array[5] * array2[12] + array[6] * array2[11] + array[7] * array2[10] + array[8] * array2[9] + array[9] * array2[8] + array[10] * array2[7] + array[11] * array2[6] + array[12] * array2[5] + array[13] * array2[4], array[5] * array2[13] + array[6] * array2[12] + array[7] * array2[11] + array[8] * array2[10] + array[9] * array2[9] + array[10] * array2[8] + array[11] * array2[7] + array[12] * array2[6] + array[13] * array2[5], array[6] * array2[13] + array[7] * array2[12] + array[8] * array2[11] + array[9] * array2[10] + array[10] * array2[9] + array[11] * array2[8] + array[12] * array2[7] + array[13] * array2[6], array[7] * array2[13] + array[8] * array2[12] + array[9] * array2[11] + array[10] * array2[10] + array[11] * array2[9] + array[12] * array2[8] + array[13] * array2[7], array[8] * array2[13] + array[9] * array2[12] + array[10] * array2[11] + array[11] * array2[10] + array[12] * array2[9] + array[13] * array2[8], array[9] * array2[13] + array[10] * array2[12] + array[11] * array2[11] + array[12] * array2[10] + array[13] * array2[9], array[10] * array2[13] + array[11] * array2[12] + array[12] * array2[11] + array[13] * array2[10], array[11] * array2[13] + array[12] * array2[12] + array[13] * array2[11], array[12] * array2[13] + array[13] * array2[12], array[13] * array2[13]);
    }
    
    @Override
    protected void reduce(long[] array) {
        this.carryReduce(array, array[0], array[1], array[2], array[3], array[4], array[5], array[6], array[7], array[8], array[9], array[10], array[11], array[12], array[13]);
    }
    
    @Override
    protected void square(long[] array, long[] array2) {
        this.carryReduce(array2, array[0] * array[0], 2L * (array[0] * array[1]), 2L * (array[0] * array[2]) + array[1] * array[1], 2L * (array[0] * array[3] + array[1] * array[2]), 2L * (array[0] * array[4] + array[1] * array[3]) + array[2] * array[2], 2L * (array[0] * array[5] + array[1] * array[4] + array[2] * array[3]), 2L * (array[0] * array[6] + array[1] * array[5] + array[2] * array[4]) + array[3] * array[3], 2L * (array[0] * array[7] + array[1] * array[6] + array[2] * array[5] + array[3] * array[4]), 2L * (array[0] * array[8] + array[1] * array[7] + array[2] * array[6] + array[3] * array[5]) + array[4] * array[4], 2L * (array[0] * array[9] + array[1] * array[8] + array[2] * array[7] + array[3] * array[6] + array[4] * array[5]), 2L * (array[0] * array[10] + array[1] * array[9] + array[2] * array[8] + array[3] * array[7] + array[4] * array[6]) + array[5] * array[5], 2L * (array[0] * array[11] + array[1] * array[10] + array[2] * array[9] + array[3] * array[8] + array[4] * array[7] + array[5] * array[6]), 2L * (array[0] * array[12] + array[1] * array[11] + array[2] * array[10] + array[3] * array[9] + array[4] * array[8] + array[5] * array[7]) + array[6] * array[6], 2L * (array[0] * array[13] + array[1] * array[12] + array[2] * array[11] + array[3] * array[10] + array[4] * array[9] + array[5] * array[8] + array[6] * array[7]), 2L * (array[1] * array[13] + array[2] * array[12] + array[3] * array[11] + array[4] * array[10] + array[5] * array[9] + array[6] * array[8]) + array[7] * array[7], 2L * (array[2] * array[13] + array[3] * array[12] + array[4] * array[11] + array[5] * array[10] + array[6] * array[9] + array[7] * array[8]), 2L * (array[3] * array[13] + array[4] * array[12] + array[5] * array[11] + array[6] * array[10] + array[7] * array[9]) + array[8] * array[8], 2L * (array[4] * array[13] + array[5] * array[12] + array[6] * array[11] + array[7] * array[10] + array[8] * array[9]), 2L * (array[5] * array[13] + array[6] * array[12] + array[7] * array[11] + array[8] * array[10]) + array[9] * array[9], 2L * (array[6] * array[13] + array[7] * array[12] + array[8] * array[11] + array[9] * array[10]), 2L * (array[7] * array[13] + array[8] * array[12] + array[9] * array[11]) + array[10] * array[10], 2L * (array[8] * array[13] + array[9] * array[12] + array[10] * array[11]), 2L * (array[9] * array[13] + array[10] * array[12]) + array[11] * array[11], 2L * (array[10] * array[13] + array[11] * array[12]), 2L * (array[11] * array[13]) + array[12] * array[12], 2L * (array[12] * array[13]), array[13] * array[13]);
    }
    
    static {
        MODULUS = evaluateModulus();
    }
}
